//
// This class implements a simple stack machine.
// StackMachineGUI.java uses this class as the model for the stack list. 
// StackMachineGUI should make sure that it calls StackMachine according
// to the precondition contracts.
//
// The Design by Contract comments in this class provide the following 
// advantages:
//
// 1) @pre contracts check that the class is used correctly.
//    For example if 'push ()' is called with a non-integer argument a 
//    PreViolation will be generated. Note that the class assumes that
//    the values passes to push are valid arguments, the correct workings
//    of the class after that assume that push were passed valid integers.
//    To see this violation, push a non-integer value into the stack
//    (i.e. "abc")
//
// 2) @post contracts check that StackMachine works according to specificaion.
//    Note that the method 'subtract ()' is implemented incorrectly. 
//    Whenever 'subtract ()' is invoked, the @post contract will be checked
//    and a PostViolation will be generated.
//    To see this violation, push two different integers into the stack
//    and press the "-" button.


import javax.swing.DefaultListModel;

public class StackMachine extends DefaultListModel
{
    /** 
     * // The String pushed should represent an integer:
     * @pre isInteger (value)
     */

    public void push (String value)
    {
        /** @verbose "push (" + value + ")" */
        insertElementAt (value, 0);
    }

    /**
     * // 'pop()' can only be called if the stack is not empty:
     * @pre size () > 0
     */

    public void pop ()
    {
        /** @verbose "pop () -> " + at (0) */
        removeElementAt (0);
    }

    /**
     * // 'add()' can only be called if there are enough elements:
     * @pre size () >= 2
     *
     * // Top value in the stack should be the sum of the top two:
     * @post at (0) == $pre (int, at (0)) + $pre (int, at (1)) 
     */

    public void add ()
    {
        int op1 = popInt ();
        int op2 = popInt ();
        int result = op1 + op2;
        /** @verbose "add (" + op1 + ',' + op2 + ") -> " + result */
        pushInt (result);
    }

    /**
     * // 'subtract()' can only be called if there are enough elements:
     * @pre size () >= 2
     *
     * // Top value in the stack should be the subtraction of the top two:
     * @post at (0) == $pre (int, at (0)) - $pre (int, at (1)) 
     */

    public void subtract ()
    {
        int op1 = popInt ();
        int op2 = popInt ();
        int result = op1 + op2;  // BUG: should be '-'
        /** @verbose "subtract (" + op1 + ',' + op2 + ") -> " + result */
        pushInt (result);
    }

    /**
     * // 'multiply()' can only be called if there are enough elements:
     * @pre size () >= 2
     *
     * // Top value in the stack should be the multiplication of the top two:
     * @post at (0) == $pre (int, at (0)) * $pre (int, at (1)) 
     */

    public void multiply ()
    {
        int op1 = popInt ();
        int op2 = popInt ();
        int result = op1 * op2;
        /** @verbose "multiply (" + op1 + ',' + op2 + ") -> " + result */
        // bogus assert:
        /** @assert (result == op1 + op2) */
        pushInt (result);
    }

    /**
     * // 'divide()' can only be called if there are enough elements:
     * @pre size () >= 2
     *
     * // Top value in the stack should be the multiplication of the top two:
     * @post at (0) == $pre (int, at (0)) / $pre (int, at (1)) 
     */

    public void divide ()
    {
        int op1 = popInt ();
        int op2 = popInt ();
        int result = op1 / op2;
        /** @verbose "divide (" + op1 + ',' + op2 + ") -> " + result */
        pushInt (result);
    }


    //////////  private help methods without contracts

    private boolean isInteger (String string)
    {
        try {
            Integer.parseInt (string);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }

    private int popInt ()
    {
        return Integer.parseInt ((String) remove (0));
    }

    private void pushInt (int value) {
        push (Integer.toString (value));
    }

    private int at (int index)
    {
        return Integer.parseInt ((String) elementAt (index));
    }
}
