// Jtest finds two NullPointerExceptions in this class for the
// methods "getEntry" and "hash".
// (Note: for the exceptions to show the suppression category
// "Direct NullPointerExceptions" should be set to "NOT suppressed").
//
// The "fix" here is just to set those two methods to private.
// The methods are internal and so it should only be possible to
// call them by the other methods of this class.
// By making them private we can be sure that only correct arguments
// will be passed to them, so one doesn't need to check for 
// illegal arguments.


package examples.dynamic.whitebox;

public class SimpleHashtable
{
    public SimpleHashtable () {
        _table = new Entry [HASHSIZE];
    }

    public Object get (String key) {
        if (key == null)
            throw new IllegalArgumentException ("key is null");
        Entry entry = getEntry (key);
        return (entry != null)? entry._value : null;
    }

    public void put (String key, Object value) {
        if (key == null)
            throw new IllegalArgumentException ("key is null");

        Entry old_entry = getEntry (key);
        if (old_entry != null) {
            old_entry._value = value;
            return;
        }
        Entry entry = new Entry (key, value);
        int hashval = hash (key);
        entry._next = _table [hashval];
        _table [hashval] = entry;
    }

    public Entry getEntry (String key) {  // should be "private"
        Entry entry = _table [hash (key)];
        while (entry != null) {
            if (entry._key.equals (key))
                return entry;
            entry = entry._next;
        }
        return null;
    }

    public int hash (String key) {  // should be "private"
        int hashval = 0;
        for (int i = 0; i < key.length (); i++)
            hashval = key.charAt (i) + 31 * hashval;
        hashval %= HASHSIZE;
	return (hashval < 0)? -hashval : hashval;
    }

    private Entry[] _table;

    private static final int HASHSIZE = 101;

    private class Entry {
        Entry (String key, Object value) {
            _key   = key;
            _value = value;
        }
        String _key;
        Object _value;
        Entry  _next;
    }


    //////////

    public static void main (String [] args) {
        SimpleHashtable table = new SimpleHashtable ();
        table.put ("one"  , new Integer (1));
        table.put ("two"  , new Integer (2));
        table.put ("three", new Integer (3));
        System.out.println (table.get ("one"));
        System.out.println (table.get ("ten"));
    }
}
